#### Seaweed Farming Suitability Model ####
# Created by Monica Moritsch and Alex Waller
# Environmental Defense Fund, San Francisco, CA, USA
# 2024

##-----------------------------------------------------------------------------------------------------------------------------
##---------------------------------------Setting Working Directory, Folder Pathways, R Packages -------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

#Install packages
install.packages("raster")
install.packages("sf")
install.packages("viridis")

# Load packages
library(raster); library(sf); library(viridis) 

##-----------------------------------------------------------------------------------------------------------------------------
###set working directory ### set file path to Folder containing 'Inputs','Intermediate','Outputs' folders
setwd("C:/Users/Name/Documents/SeaweedGISModel/Testing")

###Create an Inputs, Intermediate, and Outputs folder within the working directory
mainDir <- getwd() # Specify main working directory
inputSubDir <- "Inputs"              # Specify subdirectories
outputSubDir <- "Outputs"           
intermediateSubDir <- "Intermediate"

# Check for and create 'Inputs' folder if it doesn't exist
if (!file.exists(file.path(mainDir, inputSubDir))){
  dir.create(file.path(mainDir, inputSubDir))
}

# Check for and create 'Intermediate' folder if it doesn't exist
if (!file.exists(file.path(mainDir, intermediateSubDir))){
  dir.create(file.path(mainDir, intermediateSubDir))
}

# Check for and create 'Outputs' folder if it doesn't exist
if (!file.exists(file.path(mainDir, outputSubDir))){
  dir.create(file.path(mainDir, outputSubDir))
}

#Specify Intermediate and Outputs folder pathways
output_path = "/Outputs" #enter folder name for outputs #make sure this folder actually exists before going further
intermed_path = "/Intermediate" #enter folder name for intermediates #make sure this folder actually exists before going further

##-----------------------------------------------------------------------------------------------------------------------------
##---------------------------------------Setting Input rasters-----------------------------------------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

#### Load Input Data (All Input rasters must be save in "Inputs" folder within Working Directory)
# Most input rasters must already be appropriately transformed (e.g., log, square root, etc.) according to the user's needs, ideally to avoid too much clustering of values at one end of the distribution. 
# Note: Do not transform the raster for distance from wind farms.

study_area = raster("Inputs/EEZ_subset_SA.tif") #add names of input files
harvest = raster("Inputs/harvest_sample.tif")
GHG_reduction = raster("Inputs/GHG_reduction_sample.tif")
basin_dist = raster("Inputs/basin_distance_sample.tif")
aragonite_sat = raster("Inputs/aragonite_sat_sample.tif")
nutrients = raster("Inputs/nutrients_sample.tif")
migratory_spp = raster("Inputs/migratory_spp_sample.tif")
shipping = raster("Inputs/shipping_sample.tif")
fishing_commercial = raster("Inputs/fishing_sample.tif")
pct_coral_reef = raster("Inputs/pct_coral_reef_sample.tif")
wind_farm_dist = raster("Inputs/wind_farm_dist_km_sample.tif")
SST_future = raster("Inputs/SST_2050s_sample.tif")

# Set wind multiplier
wind_multiplier = 0.5 #number between 0 and 1, inclusive

##-----------------------------------------------------------------------------------------------------------------------------
##---------------------------------------Matching Extent / Resolution of Rasters to Reference Raster---------------------------
##-----------------------------------------------------------------------------------------------------------------------------

# List of all rasters 
rasters <- list(study_area = study_area, harvest = harvest, GHG_reduction = GHG_reduction,
                basin_dist = basin_dist, aragonite_sat = aragonite_sat, nutrients = nutrients,
                migratory_spp = migratory_spp, shipping = shipping, fishing_commercial = fishing_commercial,
                pct_coral_reef = pct_coral_reef, wind_farm_dist = wind_farm_dist, SST_future = SST_future)

# Function to print extent and resolution of a raster
print_raster_info <- function(raster, name) {
  ext <- as.character(extent(raster))
  resolution <- paste(res(raster), collapse = " x ")
  cat(name, "Extent:", ext, "\n")
  cat(name, "Resolution:", resolution, "\n\n")
}

# Print extent and resolution of all rasters
for(name in names(rasters)) {
  print_raster_info(rasters[[name]], name)
}

##-----------------------------------------------------------------------------------------------------------------------------
#### Aligning all rasters and matching raster extent and resolution  

#Function to align a raster to the reference raster
align_raster <- function(raster_to_align, reference_raster) {
  cropped_raster <- crop(raster_to_align, extent(reference_raster)) # Crop to the extent of the reference raster
  resampled_raster <- resample(cropped_raster, reference_raster, method = "bilinear") # Resample to match the resolution of the reference raster #"bilinear" for continuous data
  return(resampled_raster)
}

## Aligning all rasters to 'study_area'
harvest_aligned = align_raster(harvest, study_area)
  harvest_aligned[is.na(study_area)] = NA
GHG_reduction_aligned = align_raster(GHG_reduction, study_area)
  GHG_reduction_aligned[is.na(study_area)] = NA
basin_dist_aligned = align_raster(basin_dist, study_area)
  basin_dist_aligned[is.na(study_area)] = NA
aragonite_sat_aligned = align_raster(aragonite_sat, study_area)
  aragonite_sat_aligned[is.na(study_area)] = NA
nutrients_aligned = align_raster(nutrients, study_area)
  nutrients_aligned[is.na(study_area)] = NA
migratory_spp_aligned = align_raster(migratory_spp, study_area)
  migratory_spp_aligned[is.na(study_area)] = NA
shipping_aligned = align_raster(shipping, study_area)
  shipping_aligned[is.na(study_area)] = NA
fishing_commercial_aligned = align_raster(fishing_commercial, study_area)
  fishing_commercial_aligned[is.na(study_area)] = NA
pct_coral_reef_aligned = align_raster(pct_coral_reef, study_area)
  pct_coral_reef_aligned[is.na(study_area)] = NA
wind_farm_dist_aligned = align_raster(wind_farm_dist, study_area)
  wind_farm_dist_aligned[is.na(study_area)] = NA
SST_future_aligned = align_raster(SST_future, study_area)
  SST_future_aligned[is.na(study_area)] = NA

##-----------------------------------------------------------------------------------------------------------------------------
#### Check all raster extent and resolution match reference raster 
  
# List of aligned rasters 
aligned_rasters_list <- list(harvest_aligned = harvest_aligned, GHG_reduction_aligned = GHG_reduction_aligned, 
                             basin_dist_aligned = basin_dist_aligned, aragonite_sat_aligned = aragonite_sat_aligned, 
                             nutrients_aligned = nutrients_aligned, migratory_spp_aligned = migratory_spp_aligned, 
                             shipping_aligned = shipping_aligned, fishing_commercial_aligned = fishing_commercial_aligned, 
                             pct_coral_reef_aligned = pct_coral_reef_aligned, wind_farm_dist_aligned = wind_farm_dist_aligned, 
                             SST_future_aligned = SST_future_aligned)

# Function to print extent and resolution of an aligned raster
print_aligned_raster_info <- function(raster, name) {
  ext <- as.character(extent(raster))
  resolution <- paste(res(raster), collapse = " x ")
  cat(name, "Aligned Extent:", ext, "\n")
  cat(name, "Aligned Resolution:", resolution, "\n\n")
}

# List to print extent / resolution for each aligned raster
for(name in names(aligned_rasters_list)) {
  print_aligned_raster_info(aligned_rasters_list[[name]], name)
}

# Function to check if 2 rasters are aligned 
check_alignment <- function(raster1, raster2) {
  if (!(extent(raster1) == extent(raster2))) {
    return(FALSE)
  }
  res1 <- res(raster1)
  res2 <- res(raster2)
  if (!all.equal(res1, res2)) {
    return(FALSE)
  }
  return(TRUE)
}

# Check if all rasters are aligned with 'study_area' and print results
for(name in names(aligned_rasters_list)) {
  if(check_alignment(aligned_rasters_list[[name]], study_area)) {
    cat(name, "is aligned.\n")
  } else {
    cat(name, "is NOT aligned.\n")
  }
}

##-----------------------------------------------------------------------------------------------------------------------------
##-------------------------------------- Favorability Score Model ------------------------------------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

#####Run model
eval_units = unique(study_area) #make list of numeric codes for each study unit
for (i in eval_units){
  harvest_clipped = harvest_aligned 
  harvest_clipped[study_area != i] = NA #clip or subset all rasters to where study area == i
  
  GHG_clipped = GHG_reduction_aligned
  GHG_clipped[study_area != i] = NA  
 
  basindist_clipped = basin_dist_aligned
  basindist_clipped[study_area != i] = NA
  
  aragonite_clipped = aragonite_sat_aligned
  aragonite_clipped[study_area != i] = NA
 
  nutrients_clipped = nutrients_aligned
  nutrients_clipped[study_area != i] = NA 
  
  migratory_clipped = migratory_spp_aligned
  migratory_clipped[study_area != i] = NA
  
  shipping_clipped = shipping_aligned
  shipping_clipped[study_area != i] = NA
  
  fishing_clipped = fishing_commercial_aligned
  fishing_clipped[study_area != i] = NA

  coral_reef_clipped = pct_coral_reef_aligned
  coral_reef_clipped[study_area != i] = NA
 
  wind_dist_clipped = wind_farm_dist_aligned
  wind_dist_clipped[study_area != i] = NA
  
  SST_clipped = SST_future_aligned
  SST_clipped[study_area != i] = NA
  
  #Scale all clipped rasters according to details in manuscript table S1
  harvest_max = max(values(harvest_clipped), na.rm = T)
  harvest_min = min(values(harvest_clipped), na.rm = T)
  harvest_scaled = (harvest_clipped - harvest_min)/(harvest_max - harvest_min)

  GHG_max = max(values(GHG_clipped), na.rm = T)
  GHG_min = min(values(GHG_clipped), na.rm = T)
  GHG_scaled = (GHG_clipped - GHG_min)/(GHG_max - GHG_min)
  
  nutrients_max = max(values(nutrients_clipped), na.rm = T)
  nutrients_min = min(values(nutrients_clipped), na.rm = T)
  nutrients_scaled = (nutrients_clipped - nutrients_min)/(nutrients_max - nutrients_min)
  
  #Variables which follow the reverse pattern
  basindist_max = max(values(basindist_clipped), na.rm = T)
  basindist_min = min(values(basindist_clipped), na.rm = T)
  basindist_scaled = (basindist_max - basindist_clipped)/(basindist_max - basindist_min)
  
  fishing_max = max(values(fishing_clipped), na.rm = T)
  fishing_min = min(values(fishing_clipped), na.rm = T)
  fishing_scaled = (fishing_max - fishing_clipped)/(fishing_max - fishing_min)
  
  migratory_max = max(values(migratory_clipped), na.rm = T)
  migratory_min = min(values(migratory_clipped), na.rm = T)
  migratory_scaled = (migratory_max - migratory_clipped)/(migratory_max - migratory_min)
  
  shipping_max = max(values(shipping_clipped), na.rm = T)
  shipping_min = min(values(shipping_clipped), na.rm = T)
  shipping_scaled = (shipping_max - shipping_clipped)/(shipping_max - shipping_min)
  
  #Variables with a special pattern
  aragonite_max = max(values(aragonite_clipped), na.rm = T)
  aragonite_min = min(values(aragonite_clipped), na.rm = T)
  arag_intermed = aragonite_clipped
  aragonite_scaled = (3-arag_intermed)/(3-aragonite_min)
  aragonite_scaled[arag_intermed > 3] = 0
  
  SST_max = max(values(SST_clipped), na.rm = T) #all temps in degrees C
  SST_min = min(values(SST_clipped), na.rm = T)
  SST_scaled = SST_clipped
  SST_scaled[(SST_clipped > 27.5) & (SST_clipped <= 35)] = (SST_clipped[(SST_clipped > 27.5) & (SST_clipped <= 35)] - 27.5)/(SST_max-27.5)
  SST_scaled[(SST_clipped > 0) & (SST_clipped < 10)] = (SST_clipped[(SST_clipped > 0) & (SST_clipped < 10)] - SST_min)/(10 - SST_min)
  SST_scaled[(SST_clipped >= 10) & (SST_clipped <= 27.5)] = 1 #assign score of 1 to areas between 10 and 27.5 degrees, inclusive
  SST_scaled[SST_clipped > 35] = 0 #assign score of 0 to areas with future SST > 35
  SST_scaled[SST_clipped < 0] = 0 #assign score of 0 to areas with future SST < 0
  
  wind_scaled <- calc(wind_dist_clipped, function(x) {
    x[(x > 10) & (x <= 16)] <- (16 - x[(x > 10) & (x <= 16)]) / 6 * wind_multiplier
    x[x <= 10] <- wind_multiplier
    x[x > 16] <- 0
    return(x)
  })
  
  coral_scaled = (coral_reef_clipped-100)/100
  
  ##-----------------------------------------------------------------------------------------------------------------------------
  # Function to calculate the mean of rasters
  mean_rasters <- function(...) {
    rasters <- list(...)
    mean_raster <- calc(stack(rasters), fun = function(x) mean(x, na.rm = TRUE))  # Calculate the mean, ignoring NA values
    return(mean_raster)
  }
  
  # Calculate Scores
  GHG_reduction_score <- mean_rasters(GHG_scaled, basindist_scaled, harvest_scaled) #use cell statistics to find mean of GHG_scaled, basindist_scaled, harvest_scaled. Output should be a raster, not a number.
  water_quality_improvement_score <- mean_rasters(aragonite_scaled, nutrients_scaled) #use cell statistics to find mean of aragonite_scaled, nutrients_scaled
  constraints_prelim <- mean_rasters(migratory_scaled, fishing_scaled, shipping_scaled, coral_scaled) #use cell statistics to find mean of migratory_scaled, fishing_scaled, shipping_scaled, coral_scaled
  constraints_score = constraints_prelim + wind_scaled
  SST_score = SST_scaled
  
  # Save category scores in the Intermediate folder
  writeRaster(GHG_reduction_score, paste0(getwd(), intermed_path, "/GHG_reduction_score_", i, ".tif"), overwrite = TRUE)
  writeRaster(water_quality_improvement_score, paste0(getwd(), intermed_path, "/water_quality_improvement_score", i, ".tif"), overwrite = TRUE)
  writeRaster(constraints_score, paste0(getwd(), intermed_path, "/constraint_avoid_score_", i, ".tif"), overwrite = TRUE)
  writeRaster(SST_score, paste0(getwd(), intermed_path, "/SST_score_", i, ".tif"), overwrite = TRUE)

  
  # Find weighted averages and calculate outputs
  weightedavA = 0.4*GHG_reduction_score + 0.2*water_quality_improvement_score + 0.2*constraints_score + 0.2*SST_score 
  weightedavB = 0.2*GHG_reduction_score + 0.4*water_quality_improvement_score + 0.2*constraints_score + 0.2*SST_score
  weightedavC = 0.2*GHG_reduction_score + 0.2*water_quality_improvement_score + 0.4*constraints_score + 0.2*SST_score
  weightedavD = 0.2*GHG_reduction_score + 0.2*water_quality_improvement_score + 0.2*constraints_score + 0.4*SST_score
  weightedavE = 0.25*GHG_reduction_score + 0.25*water_quality_improvement_score + 0.25*constraints_score + 0.25*SST_score
  
  # Save outputs 
  writeRaster(weightedavA, paste0(getwd(), output_path, "/favscoreA_", i, ".tif"), overwrite = TRUE) 
  writeRaster(weightedavB, paste0(getwd(), output_path, "/favscoreB_", i, ".tif"), overwrite = TRUE)
  writeRaster(weightedavC, paste0(getwd(), output_path, "/favscoreC_", i, ".tif"), overwrite = TRUE)
  writeRaster(weightedavD, paste0(getwd(), output_path, "/favscoreD_", i, ".tif"), overwrite = TRUE)
  writeRaster(weightedavE, paste0(getwd(), output_path, "/favscoreE_", i, ".tif"), overwrite = TRUE)
}

##-----------------------------------------------------------------------------------------------------------------------------
##---------------------------------------Merged Rasters based on Favorability Score-------------------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

### Merge output rasters into a composite with all of the units
# Function to merge rasters and save them to a file
merge_and_save <- function(file_list, output_filename) {
  if (length(file_list) > 0) {
    rasters_stack <- stack(file_list) # Create RasterStack from the file list
    merged_raster <- do.call(merge, as.list(rasters_stack)) # Merge the rasters in the stack
    writeRaster(merged_raster, filename = output_filename, format = "GTiff", overwrite = TRUE) # Save the merged raster to a file
  } else {
    cat("No files to merge for", output_filename, "\n") # Function prints message if there are no files to merge / input list is empty
  }
}

# List of rasters to be merged based on category score 
favA.list <- list.files(path = paste0(getwd(),output_path), pattern = glob2rx("favscoreA*"), full.names = TRUE)
favB.list <- list.files(path = paste0(getwd(),output_path), pattern = glob2rx("favscoreB*"), full.names = TRUE)
favC.list <- list.files(path = paste0(getwd(),output_path), pattern = glob2rx("favscoreC*"), full.names = TRUE)
favD.list <- list.files(path = paste0(getwd(),output_path), pattern = glob2rx("favscoreD*"), full.names = TRUE)
favE.list <- list.files(path = paste0(getwd(),output_path), pattern = glob2rx("favscoreE*"), full.names = TRUE)

# Merge and save rasters based on category score 
merge_and_save(favA.list, file.path(paste0(getwd(),output_path), "/ALL_favscoreA.tif"))
merge_and_save(favB.list, file.path(paste0(getwd(),output_path), "/ALL_favscoreB.tif"))
merge_and_save(favC.list, file.path(paste0(getwd(),output_path), "/ALL_favscoreC.tif"))
merge_and_save(favD.list, file.path(paste0(getwd(),output_path), "/ALL_favscoreD.tif"))
merge_and_save(favE.list, file.path(paste0(getwd(),output_path), "/ALL_favscoreE.tif"))

##-----------------------------------------------------------------------------------------------------------------------------
##--------------------------------------- Seaweed Farming Favorability Plot based Category -----------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

# Function to summarize and plot a raster
diagnose_raster <- function(raster, title = "Raster Diagnostic Plot") {
  print(summary(raster))
  breaks <- seq(0, 1, by = 0.1)
  colors <- viridis(10, direction = -1)
  plot(raster, main = title, breaks = breaks, col = colors, zlim = c(0, 1))
}

### plot merged raster outputs and quantiles
diagnose_raster(raster(file.path(getwd(), output_path, "ALL_favscoreA.tif")), "Favorability score: \nEmphasis on climate benefits")
diagnose_raster(raster(file.path(getwd(), output_path, "ALL_favscoreB.tif")), "Favorability score: \nEmphasis on water quality improvement")
diagnose_raster(raster(file.path(getwd(), output_path, "ALL_favscoreC.tif")), "Favorability score: \nEmphasis on conflict avoidance")
diagnose_raster(raster(file.path(getwd(), output_path, "ALL_favscoreD.tif")), "Favorability score: \nEmphasis on temperature suitability")
diagnose_raster(raster(file.path(getwd(), output_path, "ALL_favscoreE.tif")), "Favorability score: \nAll factors equally weighted")
### Note: If plot title is cut off, enlarge the plotting area or add more '\n' to the title string to add more lines.
