#### Cost Threshold Model ####
# Created by Monica Moritsch and Alex Waller
# Environmental Defense Fund, San Francisco, CA, USA
# 2024


##-----------------------------------------------------------------------------------------------------------------------------
##---------------------------------------Setting Working Directory, Folder Pathways, R Packages -------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

###set working directory 
setwd("C:/Users/Name/Documents/SeaweedGISModel/Testing") #change file path

#Install necessary packages
install.packages("raster"); install.packages("sf")

# Load necessary packages
library(raster); library(sf)

##-----------------------------------------------------------------------------------------------------------------------------
##---------------------------------------Setting Input raster & Cost Thesholds ------------------------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

# Specify inputs
costras = raster("Inputs/CostRaster_products_SA.tif") # Net cost raster
## Note: Cost Threshold 1 must be larger than Cost Threshold 2 (th1 > th2).
th1 = 500   # Cost threshold 1
th2 = 0      # Cost threshold 2
unit = "USD/tCO2e" #CHANGE name of units to desired currency per unit of seaweed or GHG reduction #Example: US dollars per ton of CO2 equivalent

##-----------------------------------------------------------------------------------------------------------------------------
##-------------------------------------- Plots of Areas Below Cost Thresholds -------------------------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

# Find cells below cost threshold and plot

plot(costras <= th1, col = c(NA, "blue"), main = "Seaweed Farming Area \nBelow Cost Thresholds", xlab = "° Longitude", ylab = "° Latitude", legend = FALSE)
plot(costras <= th2, col = c(NA, "lightblue"), add = T, legend = FALSE)
## Note: R sometimes offsets the alignment of the second. If the light blue color is over land, try plotting it again.

# Add a custom legend to the plot
usr <- par("usr") # Retrieves plot's user coordinates
legend("topright", legend=c(paste(" < ", th1, unit), paste(" < ", th2, unit)), 
       fill=c("blue", "lightblue"), xjust=1.5, yjust=1.5, bg='white')  # Position the legend in top right corner. Move as needed.
## Note: If legend is getting cut off, enlarge plot window to redraw

##-----------------------------------------------------------------------------------------------------------------------------
##-------------------------------------- Area Below Cost Thesholds Calculations -----------------------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

# Identifies and marks cells equal to or less than Cost threshold 1
cost1 = costras
cost1[costras <= th1] = 1
cost1[costras > th1] = NA

# Identifies and marks cells equal to or less than Cost threshold 2
cost2 = costras
cost2[costras <= th2] = 1
cost2[costras > th2] = NA

# Convert rasters to polygons
polygons1 = raster::rasterToPolygons(cost1, na.rm = TRUE, dissolve = TRUE)
polygons2 = raster::rasterToPolygons(cost2, na.rm = TRUE, dissolve = TRUE)

# Convert to 'sf' objects for area calculation
polygons1_sf = st_as_sf(polygons1)
polygons2_sf = st_as_sf(polygons2)

# Set CRS to WGS84 for area calculation
polygons1_sf = st_transform(polygons1_sf, crs = 4326) # WGS84
polygons2_sf = st_transform(polygons2_sf, crs = 4326) 

# Calculate area in km^2 and add it as a new column
polygons1_sf$area_km2 = st_area(polygons1_sf) / 1e6 # Convert from m^2 to km^2
polygons2_sf$area_km2 = st_area(polygons2_sf) / 1e6 

# Correct the unit notation for area_km2 column 
polygons1_sf$area_km2 <- as.numeric(polygons1_sf$area_km2)
polygons2_sf$area_km2 <- as.numeric(polygons2_sf$area_km2)

### Or Save the polygons as Shapefiles
st_write(polygons1_sf, "Outputs/area_below_cost1.shp")
st_write(polygons2_sf, "Outputs/area_below_cost2.shp")

##-----------------------------------------------------------------------------------------------------------------------------
##-------------------------------------- Table with Total Areas Below Cost Thresholds -----------------------------------------
##-----------------------------------------------------------------------------------------------------------------------------

#Sum area_km2 with same threshold. 
total_area_th1 = sum(polygons1_sf$area_km2)
total_area_th2 = sum(polygons2_sf$area_km2)

# Table with Total Area for each Cost Threshold
results_table <- data.frame(
  'Threshold' = c(th1, th2),
  'Area_km2' = c(total_area_th1, total_area_th2)
)
colnames(results_table)[1] = paste0("Threshold_", gsub("/","_",unit))

print(results_table)

##-----------------------------------------------------------------------------------------------
# (Optional) Export table as a CSV file into Working Directory
write.csv(results_table, "Outputs/cost_threshold_total_area_table.csv", row.names = F)
